@@ -7,15 +7,16 @@
 # See LICENSE for details.
 # --- END COPYRIGHT BLOCK ---
 
-import os
 import ldap
 import ldap.dn
+from ldap.controls import SimplePagedResultsControl
 from ldap import filter as ldap_filter
 import logging
 import json
 from functools import partial
 from lib389._entry import Entry
-from lib389._constants import DIRSRV_STATE_ONLINE, SER_ROOT_DN, SER_ROOT_PW
+from lib389._constants import DIRSRV_STATE_ONLINE
+from lib389._mapped_object_lint import DSLint, DSLints
 from lib389.utils import (
         ensure_bytes, ensure_str, ensure_int, ensure_list_bytes, ensure_list_str,
         ensure_list_int, display_log_value, display_log_data
@@ -66,6 +67,34 @@ def _gen_filter(attrtypes, values, extra=None):
     return filt
 
 
+# Define wrappers around the ldap operation to have a clear diagnostic
+def _ldap_op_s(inst, f, fname, *args, **kwargs):
+    # f.__name__ says 'inner' so the wanted name is provided as argument
+    try:
+        return f(*args, **kwargs)
+    except ldap.LDAPError as e:
+        new_desc = f"{fname}({args},{kwargs}) on instance {inst.serverid}";
+        if len(e.args) >= 1:
+            e.args[0]['ldap_request'] = new_desc
+            logging.getLogger().error(f"args={e.args}")
+        raise
+
+def _add_ext_s(inst, *args, **kwargs):
+    return _ldap_op_s(inst, inst.add_ext_s, 'add_ext_s', *args, **kwargs)
+
+def _modify_ext_s(inst, *args, **kwargs):
+    return _ldap_op_s(inst, inst.modify_ext_s, 'modify_ext_s', *args, **kwargs)
+
+def _delete_ext_s(inst, *args, **kwargs):
+    return _ldap_op_s(inst, inst.delete_ext_s, 'delete_ext_s', *args, **kwargs)
+
+def _search_ext_s(inst, *args, **kwargs):
+    return _ldap_op_s(inst, inst.search_ext_s, 'search_ext_s', *args, **kwargs)
+
+def _search_s(inst, *args, **kwargs):
+    return _ldap_op_s(inst, inst.search_s, 'search_s', *args, **kwargs)
+
+
 class DSLogging(object):
     """The benefit of this is automatic name detection, and correct application
     of level and verbosity to the object.
@@ -83,7 +112,7 @@ class DSLogging(object):
             self._log.setLevel(logging.INFO)
 
 
-class DSLdapObject(DSLogging):
+class DSLdapObject(DSLogging, DSLint):
     """A single instance of DSLdapObjects
 
     :param instance: An instance
@@ -108,7 +137,6 @@ class DSLdapObject(DSLogging):
         self._must_attributes = None
         # attributes, we don't want to compare
         self._compare_exclude = ['entryid', 'modifytimestamp', 'nsuniqueid']
-        self._lint_functions = None
         self._server_controls = None
         self._client_controls = None
         self._object_filter = '(objectClass=*)'
@@ -129,7 +157,7 @@ class DSLdapObject(DSLogging):
         :returns: Entry object
         """
 
-        return self._instance.search_ext_s(self._dn, ldap.SCOPE_BASE, self._object_filter, attrlist=["*"],
+        return _search_ext_s(self._instance,self._dn, ldap.SCOPE_BASE, self._object_filter, attrlist=["*"],
                                            serverctrls=self._server_controls, clientctrls=self._client_controls,
                                            escapehatch='i am sure')[0]
 
@@ -140,7 +168,7 @@ class DSLdapObject(DSLogging):
         """
 
         try:
-            self._instance.search_ext_s(self._dn, ldap.SCOPE_BASE, self._object_filter, attrsonly=1,
+            _search_ext_s(self._instance,self._dn, ldap.SCOPE_BASE, self._object_filter, attrsonly=1,
                                         serverctrls=self._server_controls, clientctrls=self._client_controls,
                                         escapehatch='i am sure')
         except ldap.NO_SUCH_OBJECT:
@@ -148,12 +176,25 @@ class DSLdapObject(DSLogging):
 
         return True
 
+    def search(self, scope="subtree", filter='objectclass=*'):
+        search_scope = ldap.SCOPE_SUBTREE
+        if scope == 'base':
+            search_scope = ldap.SCOPE_BASE
+        elif scope == 'one':
+            search_scope = ldap.SCOPE_ONE
+        elif scope == 'subtree':
+            search_scope = ldap.SCOPE_SUBTREE
+        return _search_ext_s(self._instance,self._dn, search_scope, filter,
+                                           serverctrls=self._server_controls,
+                                           clientctrls=self._client_controls,
+                                           escapehatch='i am sure')
+
     def display(self, attrlist=['*']):
         """Get an entry but represent it as a string LDIF
 
         :returns: LDIF formatted string
         """
-        e = self._instance.search_ext_s(self._dn, ldap.SCOPE_BASE, self._object_filter, attrlist=attrlist,
+        e = _search_ext_s(self._instance,self._dn, ldap.SCOPE_BASE, self._object_filter, attrlist=attrlist,
                                         serverctrls=self._server_controls, clientctrls=self._client_controls,
                                         escapehatch='i am sure')[0]
         return e.__repr__()
@@ -179,7 +220,7 @@ class DSLdapObject(DSLogging):
         # ensure all the keys are lowercase
         str_attrs = dict((k.lower(), v) for k, v in list(str_attrs.items()))
 
-        response = json.dumps({"type": "entry", "dn": ensure_str(self._dn), "attrs": str_attrs})
+        response = json.dumps({"type": "entry", "dn": ensure_str(self._dn), "attrs": str_attrs}, indent=4)
 
         return response
 
@@ -219,6 +260,17 @@ class DSLdapObject(DSLogging):
         # How can we be sure this returns the primary one?
         return ensure_str(self.get_attr_val(self._rdn_attribute))
 
+    def get_basedn(self):
+        """Get the suffix this entry belongs to
+        """
+        from lib389.backend import Backends
+        backends = Backends(self._instance).list()
+        for backend in backends:
+            suffix = backend.get_suffix()
+            if self._dn.endswith(suffix):
+                return suffix
+        return ""
+
     def present(self, attr, value=None):
         """Assert that some attr, or some attr / value exist on the entry.
 
@@ -234,7 +286,7 @@ class DSLdapObject(DSLogging):
             raise ValueError("Invalid state. Cannot get presence on instance that is not ONLINE")
         self._log.debug("%s present(%r) %s" % (self._dn, attr, value))
 
-        e = self._instance.search_ext_s(self._dn, ldap.SCOPE_BASE, self._object_filter, attrlist=[attr, ],
+        _search_ext_s(self._instance,self._dn, ldap.SCOPE_BASE, self._object_filter, attrlist=[attr, ],
                                         serverctrls=self._server_controls, clientctrls=self._client_controls,
                                         escapehatch='i am sure')[0]
         values = self.get_attr_vals_bytes(attr)
@@ -289,7 +341,7 @@ class DSLdapObject(DSLogging):
             else:
                 value = [ensure_bytes(arg[1])]
             mods.append((ldap.MOD_REPLACE, ensure_str(arg[0]), value))
-        return self._instance.modify_ext_s(self._dn, mods, serverctrls=self._server_controls,
+        return _modify_ext_s(self._instance,self._dn, mods, serverctrls=self._server_controls,
                                            clientctrls=self._client_controls, escapehatch='i am sure')
 
     # This needs to work on key + val, and key
@@ -433,7 +485,7 @@ class DSLdapObject(DSLogging):
         elif value is not None:
             value = [ensure_bytes(value)]
 
-        return self._instance.modify_ext_s(self._dn, [(action, key, value)],
+        return _modify_ext_s(self._instance,self._dn, [(action, key, value)],
                                            serverctrls=self._server_controls, clientctrls=self._client_controls,
                                            escapehatch='i am sure')
 
@@ -473,7 +525,7 @@ class DSLdapObject(DSLogging):
             else:
                 # Error too many items
                 raise ValueError('Too many arguments in the mod op')
-        return self._instance.modify_ext_s(self._dn, mod_list, serverctrls=self._server_controls, clientctrls=self._client_controls, escapehatch='i am sure')
+        return _modify_ext_s(self._instance,self._dn, mod_list, serverctrls=self._server_controls, clientctrls=self._client_controls, escapehatch='i am sure')
 
     def _unsafe_compare_attribute(self, other):
         """Compare two attributes from two objects. This is currently marked unsafe as it's
@@ -569,7 +621,7 @@ class DSLdapObject(DSLogging):
             raise ValueError("Invalid state. Cannot get properties on instance that is not ONLINE")
         else:
             # retrieving real(*) and operational attributes(+)
-            attrs_entry = self._instance.search_ext_s(self._dn, ldap.SCOPE_BASE, self._object_filter,
+            attrs_entry = _search_ext_s(self._instance,self._dn, ldap.SCOPE_BASE, self._object_filter,
                                                       attrlist=["*", "+"], serverctrls=self._server_controls,
                                                       clientctrls=self._client_controls, escapehatch='i am sure')[0]
             # getting dict from 'entry' object
@@ -578,12 +630,32 @@ class DSLdapObject(DSLogging):
             # This could have unforseen consequences ...
             return attrs_dict
 
+    def get_all_attrs_utf8(self, use_json=False):
+        """Get a dictionary having all the attributes of the entry
+
+        :returns: Dict with real attributes and operational attributes
+        """
+
+        self._log.debug("%s get_all_attrs" % (self._dn))
+        if self._instance.state != DIRSRV_STATE_ONLINE:
+            raise ValueError("Invalid state. Cannot get properties on instance that is not ONLINE")
+        else:
+            # retrieving real(*) and operational attributes(+)
+            attrs_entry = _search_ext_s(self._instance,self._dn, ldap.SCOPE_BASE, self._object_filter,
+                                                      attrlist=["*", "+"], serverctrls=self._server_controls,
+                                                      clientctrls=self._client_controls, escapehatch='i am sure')[0]
+            # getting dict from 'entry' object
+            r = {}
+            for (k, vo) in attrs_entry.data.items():
+                r[k] = ensure_list_str(vo)
+            return r
+
     def get_attrs_vals(self, keys, use_json=False):
         self._log.debug("%s get_attrs_vals(%r)" % (self._dn, keys))
         if self._instance.state != DIRSRV_STATE_ONLINE:
             raise ValueError("Invalid state. Cannot get properties on instance that is not ONLINE")
         else:
-            entry = self._instance.search_ext_s(self._dn, ldap.SCOPE_BASE, self._object_filter,
+            entry = _search_ext_s(self._instance,self._dn, ldap.SCOPE_BASE, self._object_filter,
                                                 attrlist=keys, serverctrls=self._server_controls,
                                                 clientctrls=self._client_controls, escapehatch='i am sure')[0]
             return entry.getValuesSet(keys)
@@ -592,7 +664,7 @@ class DSLdapObject(DSLogging):
         self._log.debug("%s get_attrs_vals_utf8(%r)" % (self._dn, keys))
         if self._instance.state != DIRSRV_STATE_ONLINE:
             raise ValueError("Invalid state. Cannot get properties on instance that is not ONLINE")
-        entry = self._instance.search_ext_s(self._dn, ldap.SCOPE_BASE, self._object_filter, attrlist=keys,
+        entry = _search_ext_s(self._instance,self._dn, ldap.SCOPE_BASE, self._object_filter, attrlist=keys,
                                             serverctrls=self._server_controls, clientctrls=self._client_controls,
                                             escapehatch='i am sure')[0]
         vset = entry.getValuesSet(keys)
@@ -611,7 +683,7 @@ class DSLdapObject(DSLogging):
         else:
             # It would be good to prevent the entry code intercepting this ....
             # We have to do this in this method, because else we ignore the scope base.
-            entry = self._instance.search_ext_s(self._dn, ldap.SCOPE_BASE, self._object_filter,
+            entry = _search_ext_s(self._instance,self._dn, ldap.SCOPE_BASE, self._object_filter,
                                                 attrlist=[key], serverctrls=self._server_controls,
                                                 clientctrls=self._client_controls, escapehatch='i am sure')[0]
             vals = entry.getValues(key)
@@ -631,7 +703,7 @@ class DSLdapObject(DSLogging):
             # In the future, I plan to add a mode where if local == true, we
             # can use get on dse.ldif to get values offline.
         else:
-            entry = self._instance.search_ext_s(self._dn, ldap.SCOPE_BASE, self._object_filter,
+            entry = _search_ext_s(self._instance,self._dn, ldap.SCOPE_BASE, self._object_filter,
                                                 attrlist=[key], serverctrls=self._server_controls,
                                                 clientctrls=self._client_controls, escapehatch='i am sure')[0]
             return entry.getValue(key)
@@ -787,11 +859,11 @@ class DSLdapObject(DSLogging):
             # Is there a way to mark this as offline and kill it
             if recursive:
                 filterstr = "(|(objectclass=*)(objectclass=ldapsubentry))"
-                ents = self._instance.search_s(self._dn, ldap.SCOPE_SUBTREE, filterstr, escapehatch='i am sure')
+                ents = _search_s(self._instance, self._dn, ldap.SCOPE_SUBTREE, filterstr, escapehatch='i am sure')
                 for ent in sorted(ents, key=lambda e: len(e.dn), reverse=True):
-                    self._instance.delete_ext_s(ent.dn, serverctrls=self._server_controls, clientctrls=self._client_controls, escapehatch='i am sure')
+                    _delete_ext_s(self._instance, ent.dn, serverctrls=self._server_controls, clientctrls=self._client_controls, escapehatch='i am sure')
             else:
-                self._instance.delete_ext_s(self._dn, serverctrls=self._server_controls, clientctrls=self._client_controls, escapehatch='i am sure')
+                _delete_ext_s(self._instance, self._dn, serverctrls=self._server_controls, clientctrls=self._client_controls, escapehatch='i am sure')
 
     def _validate(self, rdn, properties, basedn):
         """Used to validate a create request.
@@ -889,7 +961,7 @@ class DSLdapObject(DSLogging):
             # If we are running in stateful ensure mode, we need to check if the object exists, and
             # we can see the state that it is in.
             try:
-                self._instance.search_ext_s(dn, ldap.SCOPE_BASE, self._object_filter, attrsonly=1, serverctrls=self._server_controls, clientctrls=self._client_controls, escapehatch='i am sure')
+                _search_ext_s(self._instance,dn, ldap.SCOPE_BASE, self._object_filter, attrsonly=1, serverctrls=self._server_controls, clientctrls=self._client_controls, escapehatch='i am sure')
                 exists = True
             except ldap.NO_SUCH_OBJECT:
                 pass
@@ -902,7 +974,7 @@ class DSLdapObject(DSLogging):
             mods = []
             for k, v in list(valid_props.items()):
                 mods.append((ldap.MOD_REPLACE, k, v))
-            self._instance.modify_ext_s(self._dn, mods, serverctrls=self._server_controls, clientctrls=self._client_controls, escapehatch='i am sure')
+            _modify_ext_s(self._instance,self._dn, mods, serverctrls=self._server_controls, clientctrls=self._client_controls, escapehatch='i am sure')
         elif not exists:
             # This case is reached in two cases. One is we are in ensure mode, and we KNOW the entry
             # doesn't exist.
@@ -913,7 +985,7 @@ class DSLdapObject(DSLogging):
             e.update({'objectclass': ensure_list_bytes(self._create_objectclasses)})
             e.update(valid_props)
             # We rely on exceptions here to indicate failure to the parent.
-            self._instance.add_ext_s(e, serverctrls=self._server_controls, clientctrls=self._client_controls, escapehatch='i am sure')
+            _add_ext_s(self._instance, e, serverctrls=self._server_controls, clientctrls=self._client_controls, escapehatch='i am sure')
             self._log.debug('Created entry %s : %s' % (dn, display_log_data(e.data)))
             # If it worked, we need to fix our instance dn for the object's self reference. Because
             # we may not have a self reference yet (just created), it may have changed (someone
@@ -955,38 +1027,10 @@ class DSLdapObject(DSLogging):
         """
         return self._create(rdn, properties, basedn, ensure=True)
 
-    def lint(self):
-        """Override this to create a linter for a type. This means that we can detect
-        and report common administrative errors in the server from our cli and
-        rest tools.
-
-        The structure of a result is::
-
-          {
-            dsle: '<identifier>'. dsle == ds lint error. Will be a code unique to
-                                this module for the error, IE DSBLE0001.
-            severity: '[HIGH:MEDIUM:LOW]'. severity of the error.
-            items: '(dn,dn,dn)'. List of affected DNs or names.
-            detail: 'msg ...'. An explination of the error.
-            fix: 'msg ...'. Steps to resolve the error.
-          }
-
-        :returns: An array of these dicts, on None if there are no errors.
-        """
-
-        if not self._lint_functions:
-            return None
-        results = []
-        for fn in self._lint_functions:
-            result = fn()
-            if result:
-                results.append(result)
-        return results
-
 
 # A challenge of this, is how do we manage indexes? They have two naming attributes....
 
-class DSLdapObjects(DSLogging):
+class DSLdapObjects(DSLogging, DSLints):
     """The object represents the next idea: "Everything is an instance of something
     that exists in this way", i.e. we unite LDAP entries by some
     set of parameters with the object.
@@ -1030,10 +1074,11 @@ class DSLdapObjects(DSLogging):
         # functions with very little work on the behalf of the overloader
         return self._childobject(instance=self._instance, dn=dn)
 
-    def list(self):
+    def list(self, paged_search=None, paged_critical=True):
         """Get a list of children entries (DSLdapObject, Replica, etc.) using a base DN
         and objectClasses of our object (DSLdapObjects, Replicas, etc.)
 
+        :param paged_search: None for no paged search, or an int of page size to use.
         :returns: A list of children entries
         """
 
@@ -1042,20 +1087,64 @@ class DSLdapObjects(DSLogging):
         # This will yield and & filter for objectClass with as many terms as needed.
         filterstr = self._get_objectclass_filter()
         self._log.debug('list filter = %s' % filterstr)
-        try:
-            results = self._instance.search_ext_s(
-                base=self._basedn,
-                scope=self._scope,
-                filterstr=filterstr,
-                attrlist=self._list_attrlist,
-                serverctrls=self._server_controls, clientctrls=self._client_controls,
-                escapehatch='i am sure'
-            )
-            # def __init__(self, instance, dn=None):
+
+        if type(paged_search) == int:
+            self._log.debug('listing with paged search -> %d', paged_search)
+            # If paged_search ->
+            results = []
+            pages = 0
+            pctrls = []
+            req_pr_ctrl = SimplePagedResultsControl(paged_critical, size=paged_search, cookie='')
+            if self._server_controls is not None:
+                controls = [req_pr_ctrl] + self._server_controls
+            else:
+                controls = [req_pr_ctrl]
+            while True:
+                msgid = self._instance.search_ext(
+                        base=self._basedn,
+                        scope=self._scope,
+                        filterstr=filterstr,
+                        attrlist=self._list_attrlist,
+                        serverctrls=controls,
+                        clientctrls=self._client_controls,
+                        escapehatch='i am sure'
+                    )
+                self._log.info('Getting page %d' % (pages,))
+                rtype, rdata, rmsgid, rctrls = self._instance.result3(msgid, escapehatch='i am sure')
+                results.extend(rdata)
+                pages += 1
+                self._log.debug("%s" % rctrls)
+                pctrls = [ c for c in rctrls
+                    if c.controlType == SimplePagedResultsControl.controlType]
+                if pctrls and pctrls[0].cookie:
+                    req_pr_ctrl.cookie = pctrls[0].cookie
+                    if self._server_controls is not None:
+                        controls = [req_pr_ctrl] + self._server_controls
+                    else:
+                        controls = [req_pr_ctrl]
+                else:
+                    break
+                #End while
+            # Result3 doesn't map through Entry, so we have to do it manually.
+            results = [Entry(r) for r in results]
             insts = [self._entry_to_instance(dn=r.dn, entry=r) for r in results]
-        except ldap.NO_SUCH_OBJECT:
-            # There are no objects to select from, se we return an empty array
-            insts = []
+            # End paged search
+        else:
+            # If not paged
+            try:
+                results = _search_ext_s(self._instance,
+                    base=self._basedn,
+                    scope=self._scope,
+                    filterstr=filterstr,
+                    attrlist=self._list_attrlist,
+                    serverctrls=self._server_controls, clientctrls=self._client_controls,
+                    escapehatch='i am sure'
+                )
+                # def __init__(self, instance, dn=None):
+                insts = [self._entry_to_instance(dn=r.dn, entry=r) for r in results]
+            except ldap.NO_SUCH_OBJECT:
+                # There are no objects to select from, se we return an empty array
+                insts = []
         return insts
 
     def exists(self, selector=[], dn=None):
@@ -1111,7 +1200,7 @@ class DSLdapObjects(DSLogging):
         filterstr = self._get_objectclass_filter()
         self._log.debug('_gen_dn filter = %s' % filterstr)
         self._log.debug('_gen_dn dn = %s' % dn)
-        return self._instance.search_ext_s(
+        return _search_ext_s(self._instance,
             base=dn,
             scope=ldap.SCOPE_BASE,
             filterstr=filterstr,
@@ -1126,7 +1215,7 @@ class DSLdapObjects(DSLogging):
         # This will yield and & filter for objectClass with as many terms as needed.
         filterstr = self._get_selector_filter(selector)
         self._log.debug('_gen_selector filter = %s' % filterstr)
-        return self._instance.search_ext_s(
+        return _search_ext_s(self._instance,
             base=self._basedn,
             scope=self._scope,
             filterstr=filterstr,
@@ -1188,7 +1277,7 @@ class DSLdapObjects(DSLogging):
         # Now actually commit the creation req
         return co.ensure_state(rdn, properties, self._basedn)
 
-    def filter(self, search, scope=None):
+    def filter(self, search, attrlist=None, scope=None, strict=False):
         # This will yield and & filter for objectClass with as many terms as needed.
         if search:
             search_filter = _gen_and([self._get_objectclass_filter(), search])
@@ -1196,9 +1285,11 @@ class DSLdapObjects(DSLogging):
             search_filter = self._get_objectclass_filter()
         if scope is None:
             scope = self._scope
-        self._log.debug(f'list filter = {search_filter} with scope {scope}')
+        if attrlist:
+            self._list_attrlist = attrlist
+        self._log.debug(f'list filter = {search_filter} with scope {scope} and attribute list {attrlist}')
         try:
-            results = self._instance.search_ext_s(
+            results = _search_ext_s(self._instance,
                 base=self._basedn,
                 scope=scope,
                 filterstr=search_filter,
@@ -1209,5 +1300,208 @@ class DSLdapObjects(DSLogging):
             insts = [self._entry_to_instance(dn=r.dn, entry=r) for r in results]
         except ldap.NO_SUCH_OBJECT:
             # There are no objects to select from, se we return an empty array
+            if strict:
+                raise ldap.NO_SUCH_OBJECT
             insts = []
         return insts
+
+class CompositeDSLdapObject(DSLdapObject):
+    """A virtual view as a single object that merges two entry attributes.
+       This class is not supposed to be called directly but through subclasses.
+
+    :param instance: An instance
+    :type instance: lib389.DirSrv
+    :param dn: Global dn
+    :type dn: str
+    """
+
+    def __init__(self, instance, dn=None):
+        super(CompositeDSLdapObject, self).__init__(instance, dn)
+        self._entries =[]
+        self._map = {}
+        self.log = None
+
+    def add_component(self, entry, attrs):
+        """Add a new entry as element of the composite entry.
+           An attribute may belong to different entries,
+           if that is the case:
+                the attribute is written in all entries
+                the attribute is read from last entry containing it.
+
+        :param entry: An entry object
+        :type entry: DSLdapObject
+        :param attrs: Attributes from the entry that are in the composite entry.
+        :type attrs: (str,...)
+        """
+
+        idx = len(self._entries)
+        self._entries.append(entry)
+        for attr in attrs:
+            nattr = attr.lower()
+            if nattr in self._map:
+                self._map[nattr].append(idx)
+            else:
+                self._map[nattr] = [idx,]
+
+    def _find_idx(self, attr, must_have=True):
+        """Find the entry index."""
+        nattr = ensure_str(attr).lower()
+        try:
+            return self._map[nattr]
+        except KeyError:
+            if must_have:
+                raise ValueError(f'No mapping for attribute {attr} in composite object')
+            return []
+
+    def _unsafe_raw_entry(self):
+        raise NotImplementedError("Composite objects have no raw view.")
+
+    def exists(self):
+        for entry in self._entries:
+            if not entry.exists():
+                return False
+        return True
+
+    def search(self, scope="subtree", filter='objectclass=*'):
+        raise NotImplementedError("Cannot use this method on composite objects.")
+
+    def get_basedn(self):
+        raise NotImplementedError("Cannot use this method on composite objects.")
+
+    def present(self, attr, value=None):
+        for entry in self._entries:
+            if not entry.present():
+                return False
+        return True
+
+    def _spread_set(self, dataset, set2attr_fn):
+        """Separate a set of data to a set per entry."""
+        res = []
+        for entry in self._entries:
+            res.append([])
+        for data in dataset:
+            attr = set2attr_fn(data)
+            for idx in self._find_idx(attr):
+                res[idx].append(data)
+        return res
+
+    def _spread_dict(self, datadict):
+        """Separate a dict of data where key is the attribure to a dict per entry."""
+        res = {}
+        for entry in self._entries:
+            res.append({})
+        for attr in datadict.keys():
+            for idx in self._find_idx(attr):
+                res[idx][attr] = datadict[attr]
+        return res
+
+    def replace_many(self, *args):
+        dataset = self._spread_set(*args, lambda x: x[0])
+        for idx in range(0, self._entries):
+            if len(dataset[idx]) > 0:
+                self._entries[idx].replace_many(*dataset[idx])
+
+    def ensure_attr_state(self, state):
+        # Hard to implement because some requires attributes may be common to several entries
+        raise NotImplementedError("Cannot use this method on composite objects.")
+
+    def set(self, key, value, action=ldap.MOD_REPLACE):
+        v = { ldap.MOD_REPLACE : "REPLACE",
+              ldap.MOD_ADD : "ADD",
+              ldap.MOD_DELETE: "DEL" }
+        for idx in self._find_idx(key):
+            self._entries[idx].set(key, value, action)
+
+    def apply_mods(self, mods):
+        dataset = self._spread_set(*args, lambda x: x[1])
+        for idx in range(0, self._entries):
+            if len(dataset[idx]) > 0:
+                self._entries[idx].apply_mods(*dataset[idx])
+
+    def get_all_attrs(self, use_json=False):
+        res = {}
+        idx = 0
+        for entry in self._entries:
+            attrs = entry.get_all_attrs(use_json)
+            for attr in attrs.keys():
+                if idx in self._find_idx(attr, False):
+                    res[attr] = attrs[attr]
+            idx += 1
+        return res
+
+    def get_all_attrs_utf8(self, use_json=False):
+        res = {}
+        idx = 0
+        for entry in self._entries:
+            attrs = entry.get_all_attrs_utf8(use_json)
+            for attr in attrs.keys():
+                if idx in self._find_idx(attr, False):
+                    res[attr] = attrs[attr]
+            idx += 1
+        return res
+
+    def get_attrs_vals(self, keys, use_json=False):
+        res = {}
+        dataset = self._spread_set(*args, lambda x: x)
+        for idx in range(0, self._entries):
+            if len(dataset[idx]) > 0:
+                res.update(self._entries[idx].apply_mods(*dataset[idx], use_json))
+
+    def get_attrs_vals_utf8(self, keys, use_json=False):
+        res = {}
+        dataset = self._spread_set(*args, lambda x: x)
+        for idx in range(0, self._entries):
+            if len(dataset[idx]) > 0:
+                res.update(self._entries[idx].apply_mods(*dataset[idx], use_json))
+
+    def get_attr_vals(self, key, use_json=False):
+        idx = self._find_idx(key)[-1]
+        return self._entries[idx].get_attr_vals(key, use_json)
+
+    def get_attr_val(self, key, use_json=False):
+        idx = self._find_idx(key)[-1]
+        return self._entries[idx].get_attr_vals(key, use_json)
+
+    def add_values(self, values):
+        raise DeprecationWarning("Not implemented any more.")
+
+    def replace_values(self, values):
+        raise DeprecationWarning("Not implemented any more.")
+
+    def set_values(self, values, action=ldap.MOD_REPLACE):
+        raise DeprecationWarning("Not implemented any more.")
+
+    def rename(self, new_rdn, newsuperior=None, deloldrdn=True):
+        raise NotImplementedError("Cannot use this method on composite objects.")
+
+    def delete(self, recursive=False):
+        raise NotImplementedError("Cannot use this method on composite objects.")
+
+    def _validate(self, rdn, properties, basedn):
+        raise NotImplementedError("Cannot use this method on composite objects.")
+
+    def _create(self, rdn=None, properties=None, basedn=None, ensure=False):
+        raise NotImplementedError("Cannot use this method on composite objects.")
+
+    def create(self, rdn=None, properties=None, basedn=None):
+        raise NotImplementedError("Cannot use this method on composite objects.")
+
+    def ensure_state(self, rdn=None, properties=None, basedn=None):
+        raise NotImplementedError("Cannot use this method on composite objects.")
+
+    def display(self, attrlist=['*']):
+        """Get an entry but represent it as a string LDIF
+        :returns: LDIF formatted string
+        """
+        eattrs = {}
+        idx = 0
+        for entry in self._entries:
+            e = _search_ext_s(entry._instance,entry._dn, ldap.SCOPE_BASE, entry._object_filter, attrlist=attrlist,
+                                        serverctrls=entry._server_controls, clientctrls=entry._client_controls,
+                                        escapehatch='i am sure')[0]
+            for attr, vals in e.iterAttrs():
+                if idx in self._find_idx(attr, False):
+                    eattrs[attr] = vals
+            idx += 1
+        e = Entry((self.dn, eattrs))
+        return e.__repr__()
