@@ -7,7 +7,7 @@
 # See LICENSE for details.
 # --- END COPYRIGHT BLOCK ---
 
-from lib389.backend import Backend, Backends, DatabaseConfig
+from lib389.backend import Backend, Backends, DatabaseConfig, BackendSuffixView
 from lib389.configurations.sample import (
     create_base_domain,
     create_base_org,
@@ -40,7 +40,6 @@ arg_to_attr = {
         'directory': 'nsslapd-directory',
         'dbcachesize': 'nsslapd-dbcachesize',
         'logdirectory': 'nsslapd-db-logdirectory',
-        'durable_txn': 'nsslapd-db-durable-transaction',
         'txn_wait': 'nsslapd-db-transaction-wait',
         'checkpoint_interval': 'nsslapd-db-checkpoint-interval',
         'compactdb_interval': 'nsslapd-db-compactdb-interval',
@@ -108,7 +107,12 @@ def _get_backend(inst, name):
     for be in be_insts:
         be_suffix = ensure_str(be.get_attr_val_utf8_l('nsslapd-suffix')).lower()
         cn = ensure_str(be.get_attr_val_utf8_l('cn')).lower()
-        if str2dn(be_suffix) == str2dn(name.lower()) or cn == name.lower():
+        try:
+            if str2dn(be_suffix) == str2dn(name.lower()):
+                return be
+        except ldap.DECODING_ERROR:
+            pass
+        if cn == name.lower():
             return be
 
     raise ValueError('Could not find backend suffix: {}'.format(name))
@@ -159,16 +163,14 @@ def backend_list(inst, basedn, log, args):
 def backend_get(inst, basedn, log, args):
     rdn = _get_arg(args.selector, msg="Enter %s to retrieve" % RDN)
     be = _get_backend(inst, rdn)
-    be_state = be.get_state()
+    bev = BackendSuffixView(inst, be)
     if args.json:
-        entry = be.get_all_attrs_json()
+        entry = bev.get_all_attrs_json()
         entry_dict = json.loads(entry)
-        entry_dict['attrs']['nsslapd-state'] = [be_state]
         log.info(json.dumps(entry_dict, indent=4))
     else:
-        entry = be.display()
+        entry = bev.display()
         updated_entry = entry[:-1]  # remove \n
-        updated_entry += "nsslapd-state: " + be_state
         log.info(updated_entry)
 
 
@@ -469,35 +471,47 @@ def backend_set(inst, basedn, log, args):
     if args.enable and args.disable:
         raise ValueError("You can not enable and disable a backend at the same time")
     if args.enable_readonly and args.disable_readonly:
-        raise ValueError("You can not set the backend to be both enabled and disabled at the same time")
+        raise ValueError("You can not set the backend readonly mode to be both enabled and disabled at the same time")
+    if args.enable_orphan and args.disable_orphan:
+        raise ValueError("You can not set the backend orphan mode to be both enabled and disabled at the same time")
 
     # Update backend
+    need_restart = False
     be = _get_backend(inst, args.be_name)
+    bev = BackendSuffixView(inst, be)
     if args.enable_readonly:
-        be.set('nsslapd-readonly', 'on')
+        bev.set('nsslapd-readonly', 'on')
     if args.disable_readonly:
-        be.set('nsslapd-readonly', 'off')
+        bev.set('nsslapd-readonly', 'off')
+    if args.enable_orphan:
+        bev.set('orphan', 'true')
+        need_restart = True
+    if args.disable_orphan:
+        bev.set('orphan', 'false')
+        need_restart = True
     if args.add_referral:
-        be.add('nsslapd-referral', args.add_referral)
+        bev.add('nsslapd-referral', args.add_referral)
     if args.del_referral:
-        be.remove('nsslapd-referral', args.del_referral)
+        bev.remove('nsslapd-referral', args.del_referral)
     if args.cache_size:
-        be.set('nsslapd-cachesize', args.cache_size)
+        bev.set('nsslapd-cachesize', args.cache_size)
     if args.cache_memsize:
-        be.set('nsslapd-cachememsize', args.cache_memsize)
+        bev.set('nsslapd-cachememsize', args.cache_memsize)
     if args.dncache_memsize:
-        be.set('nsslapd-dncachememsize', args.dncache_memsize)
+        bev.set('nsslapd-dncachememsize', args.dncache_memsize)
     if args.require_index:
-        be.set('nsslapd-require-index', 'on')
+        bev.set('nsslapd-require-index', 'on')
     if args.ignore_index:
-        be.set('nsslapd-require-index', 'off')
+        bev.set('nsslapd-require-index', 'off')
     if args.state:
-        be.set_state(args.state)
+        bev.set_state(args.state)
     if args.enable:
         be.enable()
     if args.disable:
         be.disable()
     log.info("The backend configuration was successfully updated")
+    if need_restart:
+        log.warn("Warning! The server instance must be restarted to take in account that configuration change.")
 
 
 def db_config_get(inst, basedn, log, args):
@@ -851,6 +865,8 @@ def create_parser(subparsers):
     set_backend_parser.set_defaults(func=backend_set)
     set_backend_parser.add_argument('--enable-readonly', action='store_true', help='Enables read-only mode for the backend database')
     set_backend_parser.add_argument('--disable-readonly', action='store_true', help='Disables read-only mode for the backend database')
+    set_backend_parser.add_argument('--enable-orphan', action='store_true', help='Disconnect a subsuffix from its parent suffix.')
+    set_backend_parser.add_argument('--disable-orphan', action='store_true', help='Let the subsuffix be connected to its parent suffix.')
     set_backend_parser.add_argument('--require-index', action='store_true', help='Allows only indexed searches')
     set_backend_parser.add_argument('--ignore-index', action='store_true', help='Allows all searches even if they are unindexed')
     set_backend_parser.add_argument('--add-referral', help='Adds an LDAP referral to the backend')
@@ -1015,7 +1031,6 @@ def create_parser(subparsers):
     set_db_config_parser.add_argument('--directory', help='Specifies absolute path to database instance')
     set_db_config_parser.add_argument('--dbcachesize', help='Specifies the database index cache size in bytes')
     set_db_config_parser.add_argument('--logdirectory', help='Specifies the path to the directory that contains the database transaction logs')
-    set_db_config_parser.add_argument('--durable-txn', help='Enables or disables whether database transaction log entries are immediately written to the disk')
     set_db_config_parser.add_argument('--txn-wait', help='Sets whether the server should should wait if there are no db locks available')
     set_db_config_parser.add_argument('--checkpoint-interval', help='Sets the amount of time in seconds after which the server sends a '
                                                                     'checkpoint entry to the database transaction log')
@@ -1058,7 +1073,7 @@ def create_parser(subparsers):
                                                                   'WARNING: This parameter can trigger experimental code.')
     set_db_config_parser.add_argument('--deadlock-policy', help='Adjusts the backend database deadlock policy (Advanced setting)')
     set_db_config_parser.add_argument('--db-home-directory', help='Sets the directory for the database mmapped files (Advanced setting)')
-    set_db_config_parser.add_argument('--db_lib', help='Sets which db lib is used. Valid values are: bdb or mdb')
+    set_db_config_parser.add_argument('--db-lib', help='Sets which db lib is used. Valid values are: bdb or mdb')
 
     #######################################################
     # Database & Suffix Monitor
